@@ -228,6 +228,7 @@
 #if 1
     #define NVIDIA_NFORCE           _X(NVIDIA_NFORCE)
     #define NVIDIA_NFORCE2          _X(NVIDIA_NFORCE2)
+    #define NVIDIA_NFORCE3          _X(NVIDIA_NFORCE3)
     #define NVIDIA_GENERIC          _X(NVIDIA_GENERIC)
 #endif
 
@@ -432,7 +433,7 @@ typedef struct _aper_size_info_fixed {
 	int num_entries;
 	int page_order;
 } aper_size_info_fixed;
-#endif
+#endif // 0
 
 struct agp_bridge_data {
 	agp_version *version;
@@ -442,8 +443,8 @@ struct agp_bridge_data {
 	void *dev_private_data;
 	struct pci_dev *dev;
 	struct gatt_mask *masks;
-	unsigned long *gatt_table;
-	unsigned long *gatt_table_real;
+	u32 *gatt_table;
+	u32 *gatt_table_real;
 	unsigned long scratch_page;
 	unsigned long gart_bus_addr;
 	unsigned long gatt_bus_addr;
@@ -880,6 +881,13 @@ struct agp_bridge_data {
     #define PCI_DEVICE_ID_NVIDIA_NFORCE2            0x01e0
     #endif
 
+    #ifndef PCI_DEVICE_ID_NVIDIA_NFORCE3
+    #define PCI_DEVICE_ID_NVIDIA_NFORCE3		0x00d1
+    #endif
+    #ifndef PCI_DEVICE_ID_NVIDIA_NFORCE3S
+    #define PCI_DEVICE_ID_NVIDIA_NFORCE3S		0x00e1
+    #endif
+
 #ifdef FGL
 /* pci spec conforming register offsets */
 #define PCI_VENDOR_ID       0x00
@@ -1041,7 +1049,12 @@ struct agp_bridge_data {
 #define AMD_X86_64_GARTAPERTUREBASE	0x94
 #define AMD_X86_64_GARTTABLEBASE	0x98
 #define AMD_X86_64_GARTCACHECTL		0x9c
-#define AMD_X86_64_GARTEN	1<<0
+#define AMD_X86_64_GARTEN	        (1<<0)
+/* Aperture control register bits. */
+#define GARTEN				(1<<0)
+#define DISGARTCPU			(1<<4)
+#define DISGARTIO			(1<<5)
+
 
 #define AMD_8151_VMAPERTURE		0x10
 #define AMD_8151_AGP_CTL		0xb0
@@ -1081,6 +1094,25 @@ struct agp_bridge_data {
 #define SVWRKS_POSTFLUSH  0x14                                                     
 #define SVWRKS_DIRFLUSH   0x0c
 
+/* NVIDIA registers */
+#define NVIDIA_0_APBASE     0x10
+#define NVIDIA_0_APSIZE     0x80
+#define NVIDIA_1_WBC        0xf0
+#define NVIDIA_2_GARTCTRL   0xd0
+#define NVIDIA_2_APBASE     0xd8
+#define NVIDIA_2_APLIMIT    0xdc
+#define NVIDIA_2_ATTBASE(i) (0xe0 + (i) * 4)
+#define NVIDIA_3_APBASE     0x50
+#define NVIDIA_3_APLIMIT    0x54
+
+/* NVIDIA x86-64 registers */
+#define NVIDIA_X86_64_0_APBASE		0x10
+#define NVIDIA_X86_64_1_APBASE1		0x50
+#define NVIDIA_X86_64_1_APLIMIT1	0x54
+#define NVIDIA_X86_64_1_APSIZE		0xa8
+#define NVIDIA_X86_64_1_APBASE2		0xd8
+#define NVIDIA_X86_64_1_APLIMIT2	0xdc
+
 /* HP ZX1 SBA registers */
 #define HP_ZX1_CTRL		0x200
 #define HP_ZX1_IBASE		0x300
@@ -1124,7 +1156,7 @@ extern int __init nvidia_nforce_setup(struct pci_dev *pdev);
 #endif
 
 #if LINUX_VERSION_CODE > 0x020500
-#if defined(__i386__)
+#if defined(__i386__) || defined(__x86_64__)
 #define flush_agp_cache() asm volatile("wbinvd":::"memory")
 #endif /* defined(__i386__) */
 #endif /* LINUX_VERSION_CODE > 0x020500 */
