@@ -38,7 +38,8 @@
 #endif
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,71)
-#if !defined(CONFIG_X86_PC)
+#if !defined(CONFIG_X86_PC) 
+#if !defined(CONFIG_X86_64)
 #if !defined(CONFIG_X86_VOYAGER)
 #if !defined(CONFIG_X86_NUMAQ)
 #if !defined(CONFIG_X86_SUMMIT)
@@ -53,6 +54,7 @@
 #endif
 #endif
 #endif
+#endif
 #endif /* LINUX_VERSION_CODE */
 
 // ============================================================
@@ -120,16 +122,31 @@
 #define EXPORT_NO_SYMBOLS
 #endif
 
+#ifdef __x86_64__
+#include "asm/ioctl32.h"
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,2)
+#include "linux/syscalls.h"
+#endif
+#endif
+
 #include "firegl_public.h"
 
 // ============================================================
-
+// pte_offset_map will be used by the kernel (2.4.21-15EL) come with RHEL3.0. 
+// This is for mapping pte with highmem enabled.
+// With this call, pte_unmap has to be called for releasing the atomic lock.
+// For vanilla 2.4 kernels, pte_offset is defined, pte_offset_map and 
+// pte_unmap are not.
 #ifndef pte_offset
     #ifdef pte_offset_map
         #define pte_offset  pte_offset_map
     #endif
 #endif
 
+#ifndef pte_unmap
+#define pte_unmap
+#endif
+
 #ifndef VMALLOC_VMADDR
 #define VMALLOC_VMADDR(x)  ((unsigned long)(x))
 #endif
@@ -148,6 +165,7 @@
 #define FGL_ATOMIC_PTE
 #endif
 
+
 #ifdef FGL_USE_SCT
 // get direct function pointers from sys_call_table for calling
 #else /* FGL_USE_SCT */
@@ -186,10 +204,15 @@ _syscall3( int, modify_ldt, int, func, void *, ptr, unsigned long, bytecount )
 #define FGL_VMA_API_PASS        /* none */
 #endif /* FGL_253P1_VMA_API */
 
-#ifndef VMALLOC_VMADDR
-#define VMALLOC_VMADDR(x) ((unsigned long)(x))
+#ifndef preempt_disable
+#define preempt_disable()
+#define preempt_enable()
 #endif
 
+#ifndef pte_offset_map 
+#define pte_offset_map pte_offset
+#define pte_unmap(pte)
+#endif
 // ============================================================
 /* globals */
 
@@ -273,9 +296,6 @@ static device_t     firegl_public_device;   // The Fire GL public device
 
 #define DRM(x) FGLDRM_##x
 
-#ifdef FIREGL_410
-// X4.1.0 or higher
-
 #include "drm.h"
 
 // mem_info() is missing in drm_proc.h. But, it is a DRM design problem anyway!
@@ -286,26 +306,14 @@ static int DRM(mem_info)(char *buf __attribute__((unused)), char **start __attri
     *eof = 1;
     return 0;
 }
-
+#if 0
 #ifdef vmalloc_to_page
 #undef vmalloc_to_page
 #endif
 #define vmalloc_to_page     drm_inline_vmalloc_to_page
+#endif
 #include "drm_proc.h"
 
-#else /* FIREGL_410 */
-// X4.0.3 or lower
-
-struct drm_proc_list {
-        const char *name;
-        int        (*f)(char *, char **, off_t, int, int *, void *);
-} DRM(proc_list)[] = {
-        { "",    0    },
-};
-#define DRM_PROC_ENTRIES 0
-
-#endif /* FIREGL_410 */
-
 #ifndef DRM_MAJOR
 #define DRM_MAJOR       226
 #endif // DRM_MAJOR
@@ -352,6 +360,10 @@ struct firegl_drm_stub_info_t *firegl_stub_pointer = NULL;
 #define DRM_AGP_MODULE_GET      (drm_agp_t *)inter_module_get("drm_agp")
 #define DRM_AGP_MODULE_PUT      inter_module_put("drm_agp")
 
+unsigned long ATI_API_CALL __ke_cpu_to_le32(unsigned long _u)
+{
+    return cpu_to_le32(_u);
+}
 
 static struct proc_dir_entry *firegl_proc_init( device_t *dev,
                                                 int minor,
@@ -684,7 +696,7 @@ typedef wait_queue_t*       wait_queue_head_t;
 __ke_wait_queue_head_t* ATI_API_CALL __ke_alloc_wait_queue_head_struct(void)
 {
     __ke_wait_queue_head_t* queue_head;
-    queue_head = kmalloc(sizeof(wait_queue_head_t), GFP_KERNEL);
+    queue_head = kmalloc(sizeof(wait_queue_head_t), GFP_ATOMIC);
 
     if (queue_head)
     {
@@ -788,7 +800,7 @@ void ATI_API_CALL __ke_configure_sigmask(__ke_sigset_t *pSigMask)
     sigaddset((sigset_t*)(void *)pSigMask, SIGTTOU);
 }
 
-void ATI_API_CALL __ke_block_all_signals(int (*notifier)(void *priv), void *pPriv, __ke_sigset_t *pSigMask)
+void ATI_API_CALL __ke_block_all_signals(int (*ATI_API_CALL notifier)(void *priv), void *pPriv, __ke_sigset_t *pSigMask)
 {
 #if LINUX_VERSION_CODE >= 0x020400
     block_all_signals(notifier,pPriv,(sigset_t*)(void *)pSigMask);
@@ -1034,6 +1046,84 @@ int ATI_API_CALL __ke_error_code(enum __ke_error_num errcode)
     }
 }
 
+/*****************************************************************************/
+#ifdef __x86_64__
+int ATI_API_CALL firegl_call_ioctl(unsigned int fd, unsigned int cmd, void *data)
+{
+  mm_segment_t seg;
+  int err;
+
+  seg = get_fs();
+  set_fs(KERNEL_DS);
+  err = sys_ioctl(fd, cmd, (unsigned long)data);
+  set_fs(seg);
+
+  return err;
+}
+
+int ATI_API_CALL __ke_sys_ioctl(unsigned int fd, unsigned int cmd, void *data)
+{
+  return sys_ioctl(fd, cmd, (unsigned long)data);
+}
+
+int ATI_API_CALL firegl_get_user_ptr(u32 *src, void **dst)
+{
+  void *temp=NULL;
+  int err = get_user(temp, src); 
+  *dst = temp;
+  return err;
+}
+
+int ATI_API_CALL firegl_get_user_u16(u16 *src, u16 *dst)
+{
+  u16 temp;
+  int err = get_user(temp, src);
+  *dst = temp;
+  return err;
+}
+
+int ATI_API_CALL firegl_get_user_u32(u32 *src, u32 *dst)
+{
+  u32 temp;
+  int err = get_user(temp, src);
+  *dst = temp;
+  return err;
+}
+
+int ATI_API_CALL firegl_get_user_u64(u32 *src, u64 *dst)
+{
+  u64 temp;
+  int err = get_user(temp, src);
+  *dst = temp;
+  return err;
+}
+
+int ATI_API_CALL firegl_put_user_ptr(void *src, u32 *dst)
+{
+  void *temp = src;
+  return put_user(temp, dst);
+}
+
+int ATI_API_CALL firegl_put_user_u16(u16 src, u16 *dst)
+{
+  u16 temp = src;
+  return put_user(temp, dst);
+}
+
+int ATI_API_CALL firegl_put_user_u32(u32 src, u32 *dst)
+{
+  u32 temp = src;
+  return put_user(temp, dst);
+}
+
+int ATI_API_CALL firegl_put_user_u64(u64 src, u32 *dst)
+{
+  u64 temp = src;
+  return put_user(temp, dst);
+}
+#endif /* __x86_64__ */
+
+
 /*****************************************************************************/
 
 void ATI_API_CALL __ke_mod_inc_use_count(void)
@@ -1405,6 +1495,10 @@ void* ATI_API_CALL __ke_vmalloc(__ke_size_t size)
 {
     return vmalloc(size);
 }
+void* ATI_API_CALL __ke_vmalloc_32(__ke_size_t size)
+{
+    return vmalloc_32(size);
+}
 
 void ATI_API_CALL __ke_vfree(void* p)
 {
@@ -1431,6 +1525,35 @@ void ATI_API_CALL __ke_free_pages(void* pt, int order)
     free_pages((unsigned long)pt, order);
 }
 
+void ATI_API_CALL __ke_get_page(void* pt)
+{
+    get_page(virt_to_page((unsigned long)pt));
+}
+
+void ATI_API_CALL __ke_put_page(void* pt)
+{
+    put_page(virt_to_page((unsigned long)pt));
+}
+
+void ATI_API_CALL __ke_unlock_page(void *virt)
+{
+#if LINUX_VERSION_CODE < 0x020400
+    clear_bit(PG_locked,
+        &mem_map[MAP_NR(virt)].flags);
+#else
+    unlock_page(virt_to_page((unsigned long)virt));
+#endif
+}
+
+int ATI_API_CALL __ke_PageCompound(void *virt)
+{
+#ifdef PageCompound
+    return PageCompound(virt_to_page((unsigned long)virt));
+#else
+    return 0;
+#endif
+}
+
 void ATI_API_CALL __ke_mem_map_reserve(void* pt)
 {
 #if LINUX_VERSION_CODE < 0x020400
@@ -1479,6 +1602,16 @@ void ATI_API_CALL __ke_virt_unreserve(void* virt)
 #endif
 }
 
+void ATI_API_CALL __ke_lock_page(void* virt)
+{
+#if LINUX_VERSION_CODE < 0x020400
+    set_bit(PG_locked,
+        &mem_map[MAP_NR(virt)].flags);
+#else
+    lock_page(virt_to_page((unsigned long)virt));
+#endif
+}
+
 #ifdef __ia64__
 void* ATI_API_CALL __ke_get_vmptr( struct _agp_memory* memory )
 {
@@ -1535,10 +1668,12 @@ unsigned long ATI_API_CALL __ke_get_vm_phys_addr(struct mm_struct* mm, unsigned
 #ifndef FGL_ATOMIC_PTE
 #if LINUX_VERSION_CODE > 0x020500
     pte_p = pte_offset_kernel(pmd_p, pte_linear);
+    pte = *pte_p;       // duplicate object contents
 #else
     pte_p = pte_offset(pmd_p, pte_linear);
-#endif
     pte = *pte_p;       // duplicate object contents
+    pte_unmap(pte_p);
+#endif
 #else
     pte_p = pte_offset_atomic(pmd_p, pte_linear);
     pte = *pte_p;       // duplicate object contents
@@ -1719,6 +1854,30 @@ int ATI_API_CALL __ke_mtrr_del(int reg, unsigned long base, unsigned long size)
 #endif /* !CONFIG_MTRR */
 }
 
+#ifdef __x86_64__
+int ATI_API_CALL __ke_config_iommu(void)
+{
+#ifdef CONFIG_GART_IOMMU
+    return 1;
+#else /* !CONFIG_GART_IOMMU */
+    return 0;
+#endif /* !CONFIG_GART_IOMMU */
+}
+
+int ATI_API_CALL __ke_no_iommu(void)
+{
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,1)
+#ifdef CONFIG_GART_IOMMU
+    return no_iommu;
+#else /* !CONFIG_GART_IOMMU */
+    return 0;
+#endif /* !CONFIG_GART_IOMMU */
+#else
+    return 0;
+#endif
+}
+#endif 
+
 /*****************************************************************************/
 
 int ATI_API_CALL __ke_pci_read_config_byte(__ke_pci_dev_t* dev, __ke_u8 where, __ke_u8 *val)
@@ -1751,6 +1910,17 @@ int ATI_API_CALL __ke_pci_write_config_dword(__ke_pci_dev_t* dev, __ke_u8 where,
     return pci_write_config_dword((struct pci_dev*)(void *)dev, where, val);
 }
 
+__ke_dma_addr_t ATI_API_CALL __ke_pci_map_single (__ke_pci_dev_t *pdev, void *buffer, __ke_size_t size, int direction)
+{
+    return pci_map_single((struct pci_dev*)(void*)pdev, buffer, size, direction);
+}
+
+void ATI_API_CALL __ke_pci_unmap_single (__ke_pci_dev_t *pdev, __ke_dma_addr_t bus_addr, __ke_size_t size, int direction)
+{
+    pci_unmap_single((struct pci_dev*)(void*)pdev, bus_addr, size, direction);
+}
+
+
 /*****************************************************************************/
 
 void ATI_API_CALL __ke_outb(unsigned char value, unsigned short port)
@@ -1783,6 +1953,11 @@ int ATI_API_CALL __ke_inl(unsigned short port)
     return inl(port);
 }
 
+int ATI_API_CALL __ke_log2(unsigned long x)
+{
+   return ffz(~(x));
+}
+
 /*****************************************************************************/
 // Interrupt support
 
@@ -1798,7 +1973,7 @@ void ATI_API_CALL __ke_disable_irq(int irq)
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,71)
 int ATI_API_CALL __ke_request_irq(unsigned int irq, 
-    void (*handler)(int, void *, void *),
+    void (*ATI_API_CALL handler)(int, void *, void *),
     const char *dev_name, void *dev_id)
 {
     return request_irq(irq,
@@ -1820,7 +1995,7 @@ static irqreturn_t ke_irq_handler_wrap(int irq, void *arg1, struct pt_regs *regs
 }
 
 int ATI_API_CALL __ke_request_irq(unsigned int irq, 
-    void (*handler)(int, void *, void *),
+    void (*ATI_API_CALL handler)(int, void *, void *),
     const char *dev_name, void *dev_id)
 {
     irq_handler_func = handler;
@@ -1836,6 +2011,18 @@ void ATI_API_CALL __ke_free_irq(unsigned int irq, void *dev_id)
 }
 #endif
 
+#ifdef __x86_64__
+int ATI_API_CALL __ke_register_ioctl32_conversion(unsigned int cmd, int (*handler)(unsigned int, unsigned int, unsigned long, struct file*))
+{
+    return register_ioctl32_conversion(cmd, handler);
+}
+
+void ATI_API_CALL __ke_unregister_ioctl32_conversion(unsigned int cmd)
+{
+    unregister_ioctl32_conversion(cmd);
+}
+#endif
+
 /*****************************************************************************/
 
 #ifndef NOPAGE_SIGBUS
@@ -1857,7 +2044,6 @@ static mem_map_t* virt_to_page(unsigned long kaddr)
 }
 #endif /* LINUX_VERSION_CODE < 0x020400 */
 
-
 static __inline__ vm_nopage_ret_t do_vm_nopage(struct vm_area_struct* vma,
                                                      unsigned long address)
 {
@@ -1902,12 +2088,15 @@ static __inline__ vm_nopage_ret_t do_vm_shm_nopage(struct vm_area_struct* vma,
     pte_t* pte_p;
     pte_t  pte;
     unsigned long vma_offset;
+#ifndef __x86_64__
     unsigned long linear;
+#endif
     unsigned long pte_linear;
 #if LINUX_VERSION_CODE < 0x020400
     unsigned long kaddr;
 #endif /*  LINUX_VERSION_CODE < 0x020400 */
     mem_map_t* pMmPage;
+
     /*
         vm_start           => start of vm-area,  regular address
         vm_end             => end of vm-area,    regular address
@@ -1926,12 +2115,29 @@ static __inline__ vm_nopage_ret_t do_vm_shm_nopage(struct vm_area_struct* vma,
         Note: vm_end is not member of range but this border
         hmm, might be used when growing the VMA, not sure - keep it as it is.
      */
+
+    __KE_DEBUG3("start=0x%08lx, "
+            "end=0x%08lx, "
+            "offset=0x%08lx\n",
+            vma->vm_start,
+            vma->vm_end,
+            (unsigned long)__ke_vm_offset(vma));
+
     if (address > vma->vm_end)
         return NOPAGE_SIGBUS; /* address is out of range */
 
     /*  Calculate offset into VMA */
     vma_offset = address - vma->vm_start;
 
+#if defined(__x86_64__) 
+    /*
+        The old calculation below doesn't work with 32on64.
+        Use internal routine for the proper address. 
+        This should also apply to 32-bit address, need to test out.  
+    */
+    pte_linear = firegl_get_addr_from_vm(vma) + vma_offset;
+#else
+
     /*
         TRICK:
         We dont have to keep a correspondence list between VMA offsets and
@@ -1956,6 +2162,7 @@ static __inline__ vm_nopage_ret_t do_vm_shm_nopage(struct vm_area_struct* vma,
         Convert to pte linear address (x86 => nop)
      */
     pte_linear = VMALLOC_VMADDR(linear);
+#endif
 
     /*
         Locate responsible kernel PTE for this linear address
@@ -1988,10 +2195,12 @@ static __inline__ vm_nopage_ret_t do_vm_shm_nopage(struct vm_area_struct* vma,
 #ifndef FGL_ATOMIC_PTE
 #if LINUX_VERSION_CODE > 0x020500
     pte_p = pte_offset_kernel(pmd_p, pte_linear);
+    pte = *pte_p;       // duplicate contents
 #else
     pte_p = pte_offset(pmd_p, pte_linear);
-#endif
     pte = *pte_p;       // duplicate contents
+    pte_unmap(pte_p);
+#endif
 #else
     pte_p = pte_offset_atomic(pmd_p, pte_linear);
     pte = *pte_p;       // duplicate contents
@@ -2053,7 +2262,11 @@ static __inline__ vm_nopage_ret_t do_vm_dma_nopage(struct vm_area_struct* vma,
         Resolve the kernel (mem_map/page) address for the VMA-address
         we got queried about.
     */
+#if defined(__x86_64__)
+    kaddr = firegl_get_addr_from_vm(vma) + (address - vma->vm_start);
+#else
     kaddr = __ke_vm_offset(vma) + (address - vma->vm_start);
+#endif
     pMmPage = virt_to_page(kaddr);
 
 #if 0
@@ -2075,6 +2288,63 @@ static __inline__ vm_nopage_ret_t do_vm_dma_nopage(struct vm_area_struct* vma,
 #endif /* LINUX_VERSION_CODE < 0x020400 */
 }
 
+/** 
+ **
+ **  This routine is intented to locate the page table through the 
+ **  pagelist table created earlier in dev-> pcie
+ **/
+static __inline__ vm_nopage_ret_t do_vm_pcie_nopage(struct vm_area_struct* vma,
+                                                         unsigned long address)
+{
+
+    unsigned long vma_offset;
+    unsigned long i; 
+    mem_map_t* pMmPage;
+    struct firegl_pcie_mem* pciemem;
+    unsigned long* pagelist;
+    
+    drm_device_t *dev = (drm_device_t *)firegl_get_dev_from_vm(vma);
+    if (dev == NULL)
+    {
+        __KE_ERROR("dev is NULL\n");
+        return NOPAGE_SIGBUS;
+    }
+
+    if (address > vma->vm_end)
+    {
+        __KE_ERROR("address out of range\n");
+        return NOPAGE_SIGBUS; /* address is out of range */
+    }
+    pciemem = firegl_get_pciemem_from_addr ( vma, address);
+    if (pciemem == NULL)
+    {
+        __KE_ERROR("No pciemem found! \n");
+        return NOPAGE_SIGBUS;
+    }    
+    pagelist = firegl_get_pagelist_from_vm(vma);
+
+    if (pagelist == NULL) 
+    {
+        __KE_ERROR("No pagelist! \n");
+        return NOPAGE_SIGBUS;
+    }
+     
+    /** Find offset in  vma */
+    vma_offset = address - vma->vm_start;
+    /** Which entry in the pagelist */
+    i = vma_offset >> PAGE_SHIFT;
+    pMmPage = virt_to_page(firegl_get_pcie_pageaddr_from_vm(vma,pciemem, i));
+
+    get_page(pMmPage);
+
+    if (page_address(pMmPage) == 0x0)
+    {
+        __KE_ERROR("Invalid page address\n");
+        return NOPAGE_SIGBUS;
+    }
+    return pMmPage;
+}
+
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,0)
 
 static vm_nopage_ret_t vm_nopage(struct vm_area_struct* vma,
@@ -2136,6 +2406,12 @@ static vm_nopage_ret_t vm_dma_nopage(struct vm_area_struct* vma,
         return do_vm_dma_nopage(vma, address);
 }
 
+static vm_nopage_ret_t vm_pcie_nopage(struct vm_area_struct* vma,
+                                     unsigned long address,
+                                     int write_access)
+{  
+       return do_vm_pcie_nopage(vma, address);
+}
 
 #else   /* LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,0) */
 
@@ -2193,6 +2469,13 @@ static vm_nopage_ret_t vm_dma_nopage(struct vm_area_struct* vma,
      return do_vm_dma_nopage(vma, address);
 }
 
+static vm_nopage_ret_t vm_pcie_nopage(struct vm_area_struct* vma,
+                                     unsigned long address,
+                                     int write_access)
+{
+        return do_vm_pcie_nopage(vma, address);
+}
+
 #endif /* LINUX_VERSION_CODE > KERNEL_VERSION(2,6,0) */
 
 void* ATI_API_CALL __ke_vma_file_priv(struct vm_area_struct* vma)
@@ -2275,10 +2558,12 @@ char* ATI_API_CALL __ke_vm_phys_addr_str(struct vm_area_struct* vma,
 #ifndef FGL_ATOMIC_PTE
 #if LINUX_VERSION_CODE > 0x020500
     pte_p = pte_offset_kernel(pmd_p, pte_linear);
+    pte = *pte_p;                                                                
 #else
     pte_p = pte_offset(pmd_p, pte_linear);
-#endif
     pte = *pte_p;                                                                
+    pte_unmap(pte_p);
+#endif
 #else
     pte_p = pte_offset_atomic(pmd_p, pte_linear);                                
     pte = *pte_p;       // duplicate data
@@ -2355,6 +2640,14 @@ static struct vm_operations_struct vm_ctx_ops =
     close:   ip_drm_vm_close,
 };
 
+static struct vm_operations_struct vm_pcie_ops = 
+{
+    nopage:  vm_pcie_nopage,
+    open:    ip_drm_vm_open,
+    close:   ip_drm_vm_close,
+};
+
+
 #ifdef __AGP__BUILTIN__
 #ifndef __ia64__
 static struct vm_operations_struct vm_agp_bq_ops =
@@ -2419,26 +2712,41 @@ int ATI_API_CALL __ke_vm_map(struct file* filp,
                 __KE_DEBUG("remap_page_range failed\n");
                 return -EAGAIN;
             }
-            vma->vm_flags |= VM_SHM; /* Don't swap */
+            vma->vm_flags |= VM_SHM | VM_RESERVED; /* Don't swap */
             vma->vm_ops = &vm_ops;
             }
 			break;
 
         case __KE_SHM:
-            vma->vm_flags |= VM_SHM; /* Don't swap */
+            vma->vm_flags |= VM_SHM | VM_RESERVED; /* Don't swap */
             vma->vm_ops = &vm_shm_ops;
             break;
 
+        case __KE_SG:
+
+            pages = (vma->vm_end - vma->vm_start) >> PAGE_SHIFT;
+
+#if LINUX_VERSION_CODE <= 0x02040e /* KERNEL_VERSION(2,4,14) */
+            vma->vm_flags |= VM_LOCKED | VM_SHM; /* Don't swap */
+#else
+            vma->vm_flags |= VM_RESERVED;
+#endif
+
+            //vma->vm_flags |=  VM_SHM | VM_LOCKED; /* DDDDDDDDDDon't swap */
+            //vma->vm_mm->locked_vm += pages; /* Kernel tracks aqmount of locked pages */
+            vma->vm_ops = &vm_pcie_ops;
+            break;
+
         case __KE_CTX:
             pages = (vma->vm_end - vma->vm_start) >> PAGE_SHIFT;
-            vma->vm_flags |= VM_LOCKED | VM_SHM; /* Don't swap */
+            vma->vm_flags |= VM_LOCKED | VM_SHM | VM_RESERVED; /* Don't swap */
             vma->vm_mm->locked_vm += pages; /* Kernel tracks aqmount of locked pages */
             vma->vm_ops = &vm_ctx_ops;
             break;
 
         case __KE_PCI_BQS:
             pages = (vma->vm_end - vma->vm_start) >> PAGE_SHIFT;
-            vma->vm_flags |= VM_LOCKED | VM_SHM; /* Don't swap */
+            vma->vm_flags |= VM_LOCKED | VM_SHM | VM_RESERVED; /* Don't swap */
             vma->vm_mm->locked_vm += pages; /* Kernel tracks aqmount of locked pages */
             vma->vm_ops = &vm_pci_bq_ops;
             break;
@@ -2454,7 +2762,7 @@ int ATI_API_CALL __ke_vm_map(struct file* filp,
                 	 * with sorting out the real physical pages and mappings
                 	 * in our page fault handler
                 	 */
-				vma->vm_flags |= VM_SHM; /* Don't swap */
+				vma->vm_flags |= VM_SHM | VM_RESERVED; /* Don't swap */
 				vma->vm_ops = &vm_cant_ops;
 			}
 #else
@@ -2471,7 +2779,11 @@ int ATI_API_CALL __ke_vm_map(struct file* filp,
 					__KE_DEBUG("remap_page_range failed\n");
 					return -EAGAIN;
 				}
-				vma->vm_flags |= VM_SHM; /* Don't swap */
+#ifdef __x86_64__
+				vma->vm_flags |= VM_RESERVED;
+#else
+				vma->vm_flags |= VM_SHM | VM_RESERVED; /* Don't swap */
+#endif
 				vma->vm_ops = &vm_ops;
 			}
 #endif
@@ -2486,7 +2798,7 @@ int ATI_API_CALL __ke_vm_map(struct file* filp,
                 	 * with sorting out the real physical pages and mappings
                 	 * in our page fault handler
                 	 */
-				vma->vm_flags |= VM_SHM; /* Don't swap */
+				vma->vm_flags |= VM_SHM | VM_RESERVED; /* Don't swap */
 				vma->vm_ops = &vm_cant_agp_bq_ops;
 
 			}
@@ -2504,7 +2816,11 @@ int ATI_API_CALL __ke_vm_map(struct file* filp,
 					__KE_DEBUG("remap_page_range failed\n");
 					return -EAGAIN;
 				}
-				vma->vm_flags |= VM_SHM; /* Don't swap */
+#ifdef __x86_64__
+				vma->vm_flags |= VM_RESERVED;
+#else
+				vma->vm_flags |= VM_SHM | VM_RESERVED; /* Don't swap */
+#endif
 				vma->vm_ops = &vm_agp_bq_ops;
 			}
 #endif
@@ -3079,7 +3395,7 @@ int ATI_API_CALL __ke_smp_processor_id(void)
 }
 
 
-void ATI_API_CALL __ke_smp_call_function( void (*func)(void *info) )
+void ATI_API_CALL __ke_smp_call_function( void (*ATI_API_CALL func)(void *info) )
 {
 	smp_call_function( func, NULL, 0, 1 );
 }
