@@ -53,6 +53,8 @@ struct pci_bus;
 struct drm_device;
 struct drm_agp_mem;
 struct drm_map;
+struct firegl_pcie_mem;
+struct firegl_pcie;
 
 #if !defined(ATI_API_CALL)
 #define ATI_API_CALL __attribute__((regparm(0)))
@@ -67,6 +69,8 @@ typedef long __ke_ssize_t;
 typedef unsigned char __ke_u8;
 typedef unsigned short __ke_u16;
 typedef unsigned int __ke_u32;
+typedef unsigned long long __ke_u64;
+typedef unsigned long __ke_dma_addr_t;
 
 // note: assigning uniqe types to originally non interchangeable types
 typedef struct { int uniqe1; } __ke_wait_queue_head_t;
@@ -128,8 +132,11 @@ extern struct drm_agp_mem* ATI_API_CALL firegl_get_agpmem_from_drmmap(struct drm
 							unsigned long baddr);
 extern int ATI_API_CALL firegl_cant_use_agp(void *pt);
 extern void* ATI_API_CALL firegl_get_dev_from_vm(  struct vm_area_struct* vma );
-
-
+extern void* ATI_API_CALL firegl_get_pcie_from_vm(  struct vm_area_struct* vma );
+extern void* ATI_API_CALL firegl_get_pciemem_from_addr( struct vm_area_struct* vma, unsigned long addr );
+extern unsigned long ATI_API_CALL firegl_get_pcie_pageaddr_from_vm(  struct vm_area_struct* vma, struct firegl_pcie_mem* pciemem, unsigned long offset);
+extern void* ATI_API_CALL firegl_get_pagelist_from_vm(  struct vm_area_struct* vma );
+extern unsigned long ATI_API_CALL firegl_get_addr_from_vm(  struct vm_area_struct* vma);
 
 /*****************************************************************************/
 
@@ -148,16 +155,16 @@ extern int ATI_API_CALL __ke_signal_pending(void);
 extern void ATI_API_CALL __ke_set_current_state_task_interruptible(void);
 extern void ATI_API_CALL __ke_set_current_state_task_running(void);
 extern void ATI_API_CALL __ke_configure_sigmask(__ke_sigset_t *pSigMask);
-extern void ATI_API_CALL __ke_block_all_signals(int (*notifier)(void *priv), void *pPriv, __ke_sigset_t *pSigMask);
+extern void ATI_API_CALL __ke_block_all_signals(int (*ATI_API_CALL notifier)(void *priv), void *pPriv, __ke_sigset_t *pSigMask);
 extern void ATI_API_CALL __ke_unblock_all_signals(void);
 
 extern unsigned long ATI_API_CALL __ke__cmpxchg(volatile void *ptr, unsigned long old,                      
                       unsigned long new, int size);
+extern unsigned long ATI_API_CALL __ke_cpu_to_le32(unsigned long _u);
 
 #define __ke_cmpxchg(ptr,o,n)                        \
   ((__typeof__(*(ptr)))__ke__cmpxchg((ptr),(unsigned long)(o),      \
                  (unsigned long)(n),sizeof(*(ptr))))
-                                                                                        
 /*****************************************************************************/
 
 extern __ke_dev_t ATI_API_CALL __ke_getdevice(__ke_device_t *dev);
@@ -251,11 +258,17 @@ extern int ATI_API_CALL __ke_verify_area(int type, const void * addr, unsigned l
 extern void* ATI_API_CALL __ke_malloc(__ke_size_t size);
 extern void ATI_API_CALL __ke_free_s(void* p, __ke_size_t size);
 extern void* ATI_API_CALL __ke_vmalloc(__ke_size_t size);
+extern void* ATI_API_CALL __ke_vmalloc_32(__ke_size_t size);
 extern void ATI_API_CALL __ke_vfree(void* p);
 extern void* ATI_API_CALL __ke_get_free_page(void);
 extern void* ATI_API_CALL __ke_get_free_pages(int order);
 extern void ATI_API_CALL __ke_free_page(void* pt);
 extern void ATI_API_CALL __ke_free_pages(void* pt, int order);
+extern void ATI_API_CALL __ke_get_page(void* pt);
+extern void ATI_API_CALL __ke_put_page(void* pt);
+extern void ATI_API_CALL __ke_lock_page(void* pt);
+extern void ATI_API_CALL __ke_unlock_page(void *virt);
+extern int ATI_API_CALL __ke_PageCompound(void *virt);
 extern void ATI_API_CALL __ke_mem_map_reserve(void* pt);
 extern void ATI_API_CALL __ke_mem_map_unreserve(void* pt);
 extern void ATI_API_CALL __ke_virt_reserve(void* pt);
@@ -266,13 +279,13 @@ extern void* ATI_API_CALL __ke_ioremap_nocache(unsigned long offset, unsigned lo
 extern void ATI_API_CALL __ke_iounmap(void* pt);
 extern int ATI_API_CALL __ke_verify_read_access(void* addr, __ke_size_t size);
 extern int ATI_API_CALL __ke_verify_write_access(void* addr, __ke_size_t size);
-extern struct mm_struct* ATI_API_CALL __ke_init_mm(void);
-extern struct mm_struct* ATI_API_CALL __ke_current_mm(void);
+extern struct ATI_API_CALL mm_struct* ATI_API_CALL __ke_init_mm(void);
+extern struct ATI_API_CALL mm_struct* ATI_API_CALL __ke_current_mm(void);
 extern unsigned long ATI_API_CALL __ke_get_vm_phys_addr(struct mm_struct* mm, unsigned long virtual_addr);
 extern unsigned long* ATI_API_CALL __ke_get_vm_phys_addr_list(struct mm_struct* mm, unsigned long virtual_addr, unsigned long pages);
 extern void* ATI_API_CALL __ke_memset(void* s, int c, __ke_size_t count);
 extern void* ATI_API_CALL __ke_memcpy(void* d, const void* s, __ke_size_t count);
-extern __ke_size_t ATI_API_CALL __ke_strlen(const char *s);
+extern ATI_API_CALL __ke_size_t __ke_strlen(const char *s);
 extern char* ATI_API_CALL __ke_strcpy(char* d, const char* s);
 extern char* ATI_API_CALL __ke_strncpy(char* d, const char* s, __ke_size_t count);
 extern int ATI_API_CALL __ke_strcmp(const char *s1, const char *s2);
@@ -294,6 +307,11 @@ extern int ATI_API_CALL __ke_config_mtrr(void);
 extern int ATI_API_CALL __ke_mtrr_add_wc(unsigned long base, unsigned long size);
 extern int ATI_API_CALL __ke_mtrr_del(int reg, unsigned long base, unsigned long size);
 
+#ifdef __x86_64__
+extern int ATI_API_CALL __ke_config_iommu(void);
+extern int ATI_API_CALL __ke_no_iommu(void);
+#endif
+
 /*****************************************************************************/
 
 extern int ATI_API_CALL __ke_get_pci_device_info(__ke_pci_dev_t* dev, __ke_pci_device_info_t *pinfo);
@@ -321,7 +339,7 @@ extern int ATI_API_CALL __ke_in(unsigned short port);
 
 extern void ATI_API_CALL __ke_enable_irq(int irq);
 extern void ATI_API_CALL __ke_disable_irq(int irq);
-extern int ATI_API_CALL __ke_request_irq(unsigned int irq, void (*handler)(int, void *, void *), const char *dev_name, void *dev_id);
+extern int ATI_API_CALL __ke_request_irq(unsigned int irq, void (*ATI_API_CALL handler)(int, void *, void *), const char *dev_name, void *dev_id);
 extern void ATI_API_CALL __ke_free_irq(unsigned int irq, void *dev_id);
 
 /*****************************************************************************/
@@ -338,6 +356,7 @@ enum __ke_vm_maptype
     __KE_PCI_BQS,
     __KE_AGP_BQS,
     __KE_AGP,
+    __KE_SG,
 };
 extern char* ATI_API_CALL __ke_vm_flags_str(struct vm_area_struct* vma, char* buf);
 extern char* ATI_API_CALL __ke_vm_page_prot_str(struct vm_area_struct* vma, char* buf);
@@ -350,6 +369,15 @@ extern int ATI_API_CALL __ke_vm_map(struct file* filp,
                        enum __ke_vm_maptype type,
                        int readonly);
 
+#ifdef __x86_64__
+/*
+typedef int (*__ke_ioctl_trans_handler_t)(unsigned int, unsigned int,
+                                          unsigned long, struct file *);
+*/
+extern int ATI_API_CALL __ke_register_ioctl32_conversion(unsigned int cmd, int (*handler)(unsigned int, unsigned int, unsigned long, struct file*));
+extern void ATI_API_CALL __ke_unregister_ioctl32_conversion(unsigned int cmd);
+#endif
+
 /*****************************************************************************/
 
 typedef struct __ke_agp_version
@@ -398,7 +426,7 @@ unsigned long ATI_API_CALL __ke_agp_memory_page_count(struct _agp_memory* handle
 int ATI_API_CALL __ke_is_athlon(void);
 int ATI_API_CALL __ke_has_PSE(void);
 int ATI_API_CALL __ke_amd_adv_spec_cache_feature(void);
-void ATI_API_CALL __ke_smp_call_function( void (*func)(void *info) );
+void ATI_API_CALL __ke_smp_call_function( void (*ATI_API_CALL func)(void *info) );
 int ATI_API_CALL __ke_smp_processor_id(void);
 
 /*****************************************************************************/
@@ -419,8 +447,34 @@ void ATI_API_CALL firegl_agp_cleanup(void);
 unsigned long ATI_API_CALL firegl_agp_memory_handle(struct _agp_memory* handle);
 unsigned long ATI_API_CALL firegl_agp_memory_page_count(struct _agp_memory* handle);
 
+#ifdef __x86_64__
+int ATI_API_CALL firegl_call_ioctl(unsigned int fd, unsigned int cmd, void *data);
+int ATI_API_CALL __ke_sys_ioctl(unsigned int fd, unsigned int cmd, void *data);
+
+int ATI_API_CALL firegl_get_user_ptr(__ke_u32 *src,   void **dst);
+int ATI_API_CALL firegl_get_user_u16(__ke_u16 *src,   __ke_u16 *dst);
+int ATI_API_CALL firegl_get_user_u32(__ke_u32 *src,   __ke_u32 *dst);
+int ATI_API_CALL firegl_get_user_u64(__ke_u32 *src,   __ke_u64 *dst);
+
+int ATI_API_CALL firegl_put_user_ptr(void *src,      __ke_u32 *dst);
+int ATI_API_CALL firegl_put_user_u16(__ke_u16 src,   __ke_u16 *dst);
+int ATI_API_CALL firegl_put_user_u32(__ke_u32 src,   __ke_u32 *dst);
+int ATI_API_CALL firegl_put_user_u64(__ke_u64 src,   __ke_u32 *dst);
+#endif
+
 /*****************************************************************************/
 
+extern __ke_dma_addr_t ATI_API_CALL __ke_pci_map_single (__ke_pci_dev_t *pdev, void *buffer, __ke_size_t size, int direction); 
+extern void ATI_API_CALL __ke_pci_unmap_single (__ke_pci_dev_t *pdev, __ke_dma_addr_t bus_addr, __ke_size_t size, int direction); 
+extern int ATI_API_CALL __ke_log2(unsigned long x);
+
+
+#define __ke_PCI_DMA_BIDIRECTIONAL  0
+#define __ke_PCI_DMA_TODEVICE       1
+#define __ke_PCI_DMA_FROMDEVICE     2
+#define __ke_PCI_DMA_NONE           3
+
+
 /* globals */
 extern int __ke_debuglevel;
 #define DEBUGFLAG_DEADBEAF      0x00010000
@@ -430,12 +484,13 @@ extern int __ke_debuglevel;
 
 extern int __ke_moduleflags;
 
-/* module flags */
+/* module flags*/
 #define __KE_FLAG_DEBUG                     0x00000001
 #define __KE_FLAG_NOAUTH                    0x00000002
 #define __KE_FLAG_DISABLE_AGPGART           0x00000004
 #define __KE_FLAG_DISABLE_FIREGL_AGPGART    0x00000008
 #define __KE_FLAG_DISABLE_FIREGL_AGPLOCK    0x00000010
+#define __KE_FLAG_DISABLE_DYNAMIC_PCIE      0x00000020
 
 /* global constants */
 extern const char*          __ke_UTS_RELEASE;
