@@ -40,6 +40,33 @@ fi
 #CC=gcc -V 2.96
 #CC=gcc -V 3.2
 
+# sets the GCC to use to the one required by the module (if available)
+function set_GCC_version () {
+  #identify GCC default version major number
+  GCC_MAJOR="`gcc --version | grep -o -e "(GCC) ." | cut -d " " -f 2`"
+
+  #identify the GCC major version that compiled the kernel
+  KERNEL_GCC_MAJOR="`cat /proc/version | grep -o -e "gcc version ."  | cut -d " " -f 3`"
+
+  #see if they don't match
+  if [ ${GCC_MAJOR} != ${KERNEL_GCC_MAJOR} ]; then
+       #use kernel GCC version hopefully
+       KERNEL_GCC="`cat /proc/version | grep -o -e "gcc version .* " | cut -d " " -f 3`"
+       CC="gcc -V ${KERNEL_GCC}"
+
+       # check if gcc version requested exists
+       GCC_AVAILABLE="`${CC} --version | grep -e "(GCC)" |cut -d " " -f 3`"
+    
+       if [ ${GCC_AVAILABLE} != ${KERNEL_GCC} ]; then
+           echo "The GCC version that is required to compile this module is version ${KERNEL_GCC}."
+           echo "Please install this GCC or recompile your kernel with ${GCC_AVAILABLE}"
+           exit 1
+       fi
+  fi
+}
+
+set_GCC_version
+
 # parse options
 if [ "$1" = "--nohints" ]; then
   OPTIONS_HINTS=0
@@ -150,79 +177,12 @@ if [ -d /lib/modules/${uname_r}/build/include ];
 then
   # just comment this line out if you already set an alternative location
   linuxincludes=/lib/modules/${uname_r}/build/include
-
-  # handle SuSE 9.0 - there are generic and specialized includes
-  if [ -h /lib/modules/${uname_r}/build ]
-  then
-    build_dir=`find /lib/modules/${uname_r}/build -printf "%l\n"`
-    if [ ! -d ${build_dir}/drivers/char/drm ]
-    then
-      if [ `echo ${build_dir} | grep -c -e "-include/"` -gt 0 ]
-      then
-        build_dir_2=`echo ${build_dir} | sed -e 's/-include.*//g'`
-        drmincludes_2=${build_dir_2}/drivers/char/drm
-        if [ ! -d ${drmincludes_2} ]
-        then
-          unset build_dir_2
-          unset drmincludes_2
-        fi
-      fi
-    fi
-  fi
 fi
 
 else
   linuxincludes=${KERNEL_PATH}/include
 fi
 
-# assign default location of drm header files
-# *** adapt to your individual setup if needed ***
-drmincludes=${linuxincludes}/../drivers/char/drm
-#drmincludes=/usr/local/src/dripkg/drm
-
-# since kernel 2.4.8 the X4.1.0 drm kernel module headers are part
-# of the kernel build environment - check dir and expected files.
-
-# first assume we are facing the old DRM
-drmdefines="-DFIREGL_403"
-
-# check if we can find a few files that are only in drm from X410
-if [ -d ${drmincludes} ];
-then
-  # the directory is present, now check if needed files do exist
-  if [ -e ${drmincludes}/drm.h ]
-  then
-    if [ -e ${drmincludes}/drm_proc.h ]
-    then
-      # it could be the drm headers from X4.1.0 or higher
-      drmdefines="-DFIREGL_410"
-    fi
-  fi
-fi
-
-# if we are still at the default value
-if [ ! "$drmdefines" = "-DFIREGL_410" ];
-then
-# if we were able to find a secondary location
-if [ ! -z ${drmincludes_2} ];
-then
-# tryout the SuSE 9.0 variant
-if [ -d ${drmincludes_2} ];
-then
-  # the directory is present, now check if needed files do exist
-  if [ -e ${drmincludes_2}/drm.h ]
-  then
-    if [ -e ${drmincludes_2}/drm_proc.h ]
-    then
-      # it could be the drm headers from X4.1.0 or higher
-      drmdefines="-DFIREGL_410"
-      drmincludes=${drmincludes_2}
-    fi
-  fi
-fi
-fi
-fi
-
 # ==============================================================
 # print a few statistics, helpful for analyzing any build failures
 echo ATI module generator V 2.0 | tee $logfile
@@ -305,16 +265,6 @@ fi
 
 OsVersion=${uname_v}
 
-# check if results from drm include file checking are sufficient
-if [ ! "$drmdefines" = "-DFIREGL_410" ];
-then
-  echo "Error:"                                                         | tee -a $logfile
-  echo "XFree86 drm includes at $drmincludes do not fit this driver."   | tee -a $logfile
-  echo "This driver is designed to only work with X4.1.0 or higher."    | tee -a $logfile
-  echo "You can match this by getting Linux kernel 2.4.8 or higher."    | tee -a $logfile
-  exit 1
-fi
-
 if [ $DEMAND_BZIMAGE -gt 0 ]
 then
 
@@ -748,28 +698,6 @@ then
   rmdir patch/linux 2>/dev/null
 fi
 
-if [ -e patch/drivers/char/drm ]
-then
-  if [ -e patch/drivers/char/drm/drm_proc.h ]
-  then
-    rm -f patch/drivers/char/drm/drm_proc.h
-  fi
-  if [ -e patch/drivers/char/drm/drmP.h ]
-  then
-    rm -f patch/drivers/char/drm/drmP.h
-  fi
-  if [ -e patch/drivers/char/drm/drm_os_linux.h ]
-  then
-    rm -f patch/drivers/char/drm/drm_os_linux.h
-  fi
-  # cleanup any patchfile remainder
-  rm -f patch/drivers/char/drm/*.orig patch/drivers/char/drm/*.rej
-  # cleanup dirs
-  rmdir patch/drivers/char/drm
-  rmdir patch/drivers/char
-  rmdir patch/drivers
-fi
-
 if [ -e patch ]
 then
   rmdir patch 2>/dev/null
@@ -789,79 +717,6 @@ then
   patch_includes="${patch_includes} -Ipatch/include"
 fi
 
-srcfile=${drmincludes}/drmP.h
-if [ -e ${srcfile} ]
-then
- if [ `cat ${srcfile} | grep -c "\"[\" DRM_NAME \":\" __FUNCTION__ \"] \" fmt ,"` -eq 0 ]
- then
-  echo "skipping patch for 'drmP.h', not needed" | tee -a $logfile
-  # a local copy is still needed for propper compile of patched drm_os_linux.h
-  mkdir -p patch/drivers/char/drm
-  cd patch/drivers/char/drm
-  if [ -e ${drmincludes}/drm_proc.h ]
-  then
-    cp ${drmincludes}/drm_proc.h drm_proc.h
-  fi
-  cp ${srcfile} drmP.h
-  cd ${current_wd}
- else
-  echo "patching 'drmP.h'..." | tee -a $logfile
-  mkdir -p patch/drivers/char/drm
-  cd patch/drivers/char/drm
-
-  # we need both of below files, even if we dont apply any changes to the first.
-  # this will work around the inclusion concept and fix the gcc3.2 warnings
-  cp ${drmincludes}/drm_proc.h drm_proc.h
-  cat <<-begin_end | sed -e 's/\\x/\\/g' >drmP.h.patch
---- drmP.h.orig	2002-10-15 20:30:21.000000000 +0200
-+++ drmP.h	2002-10-15 20:31:58.000000000 +0200
-@@ -255,9 +255,9 @@
- 
- 				/* Macros to make printk easier */
- #define DRM_ERROR(fmt, arg...) \x
--	printk(KERN_ERR "[" DRM_NAME ":" __FUNCTION__ "] *ERROR* " fmt , ##arg)
-+	printk(KERN_ERR "[" DRM_NAME ":%s] *ERROR* " fmt, __FUNCTION__ , ##arg)
- #define DRM_MEM_ERROR(area, fmt, arg...) \x
--	printk(KERN_ERR "[" DRM_NAME ":" __FUNCTION__ ":%s] *ERROR* " fmt , \x
-+	printk(KERN_ERR "[" DRM_NAME ":%s:%s] *ERROR* " fmt, __FUNCTION__, \x
- 	       DRM(mem_stats)[area].name , ##arg)
- #define DRM_INFO(fmt, arg...)  printk(KERN_INFO "[" DRM_NAME "] " fmt , ##arg)
- 
-@@ -266,7 +266,7 @@
- 	do {								\x
- 		if ( DRM(flags) & DRM_FLAG_DEBUG )			\x
- 			printk(KERN_DEBUG				\x
--			       "[" DRM_NAME ":" __FUNCTION__ "] " fmt ,	\x
-+			       "[" DRM_NAME ":%s] " fmt, __FUNCTION__ ,  \x
- 			       ##arg);					\x
- 	} while (0)
- #else
-begin_end
-  
-  cp ${srcfile} drmP.h
-  patch -p0 -E <drmP.h.patch
-  rm -f drmP.h.patch
-  cd ${current_wd}
-  patch_includes="${patch_includes} -Ipatch/drivers/char/drm"
- fi
-fi
-
-srcfile=${drmincludes}/drm_os_linux.h
-if [ -e ${srcfile} ]
-then
- if [ `cat ${srcfile} | grep -c "#warning"` -eq 0 ]
- then
-  echo "skipping patch for 'drm_os_linux.h', not needed" | tee -a $logfile
- else
-  echo "patching 'drm_os_linux.h'..." | tee -a $logfile
-  mkdir -p patch/drivers/char/drm
-  cd patch/drivers/char/drm
-  cat ${srcfile} | grep -v "#warning" > drm_os_linux.h
-  cd ${current_wd}
-  patch_includes="${patch_includes} -Ipatch/drivers/char/drm"
- fi
-fi
-
 # ==============================================================
 # defines for all targets
 def_for_all="-DATI_AGP_HOOK -DATI -DFGL -D${target_define} -DFGL_CUSTOM_MODULE -DPAGE_ATTR_FIX=$PAGE_ATTR_FIX"
@@ -990,8 +845,12 @@ fi
 if [ $kernel_is_26x -gt 0 ]; then
     echo "doing Makefile based build for kernel 2.6.x and higher"   | tee -a $logfile
     cd 2.6.x
-    make PAGE_ATTR_FIX=$PAGE_ATTR_FIX 2>&1                          | tee -a $logfile
+    #tlog is a tempory file that will be deleted
+    make PAGE_ATTR_FIX=$PAGE_ATTR_FIX > tlog 2>&1 
     res=$?
+    tee -a $logfile < tlog
+    #delete tlog
+    rm -f tlog
     cd ..
     if [ $res -eq 0 ]; then
         echo "build succeeded with return value $res"               | tee -a $logfile
@@ -1103,7 +962,7 @@ fi
 SRC=${SOURCE_PREFIX}/${FGL_PUBLIC}_public.c
 DST=${FGL_PUBLIC}_public.o
 echo "compiling '$SRC'..." | tee -a $logfile
-cc_cmd="${CC} ${WARNINGS} -O2 -D__KERNEL__ -DMODULE -fomit-frame-pointer $def_for_all $def_machine -D${MODULE} $def_vma_api_version $def_smp $def_modversions $def_agp $patch_includes -I$linuxincludes -I$drmincludes $drmdefines -c $SRC -o $DST"
+cc_cmd="${CC} ${WARNINGS} -O2 -D__KERNEL__ -DMODULE -fomit-frame-pointer $def_for_all $def_machine -D${MODULE} $def_vma_api_version $def_smp $def_modversions $def_agp $patch_includes -I$linuxincludes -I$PWD -c $SRC -o $DST"
 echo "$cc_cmd" >> $logfile
 $cc_cmd 2>&1 | tee -a $logfile | grep -v "warning: pasting would not give a valid preprocessing token"
 if [ ! -e $DST ] ;
