@@ -333,8 +333,9 @@ ieee80211_dump_nodes(ic);/*XXX*/
 				 * Discard encrypted frames when privacy is off.
 				 */
 				IEEE80211_DPRINTF(ic, IEEE80211_MSG_INPUT,
-					("[%s] discard WEP frame 'cuz PRIVACY "
-					"off\n", ether_sprintf(wh->i_addr2)));
+					("[%s] discard WEP data frame 'cuz "
+					"PRIVACY off\n",
+					ether_sprintf(wh->i_addr2)));
 				ic->ic_stats.is_rx_noprivacy++;
 				goto out;
 			}
@@ -479,12 +480,43 @@ ieee80211_dump_nodes(ic);/*XXX*/
 #ifdef IEEE80211_DEBUG
 		if ((ieee80211_msg_debug(ic) && doprint(ic, subtype)) ||
 		    ieee80211_msg_dumppkts(ic)) {
-			if_printf(ic->ic_dev, "received %s from %s rssi %d\n",
-			    ieee80211_mgt_subtype_name[subtype
-			    >> IEEE80211_FC0_SUBTYPE_SHIFT],
-			    ether_sprintf(wh->i_addr2), rssi);
+			if_printf(ic->ic_dev, "[%s] received %s rssi %d\n",
+			    ether_sprintf(wh->i_addr2),
+			    ieee80211_mgt_subtype_name[subtype >>
+				IEEE80211_FC0_SUBTYPE_SHIFT], rssi);
 		}
 #endif
+		if (wh->i_fc[1] & IEEE80211_FC1_WEP) {
+			if (subtype != IEEE80211_FC0_SUBTYPE_AUTH) {
+				/*
+				 * Only shared key auth frames with a challenge
+				 * should be encrypted, discard all others.
+				 */
+				IEEE80211_DPRINTF(ic, IEEE80211_MSG_INPUT,
+					("[%s] discard %s with WEP\n",
+					ether_sprintf(wh->i_addr2),
+					ieee80211_mgt_subtype_name[subtype >>
+					    IEEE80211_FC0_SUBTYPE_SHIFT]));
+				ic->ic_stats.is_rx_mgtdiscard++; /* XXX */
+				goto out;
+			}
+			if ((ic->ic_flags & IEEE80211_F_PRIVACY) == 0) {
+				/*
+				 * Discard encrypted frames when privacy is off.
+				 */
+				IEEE80211_DPRINTF(ic, IEEE80211_MSG_INPUT,
+					("[%s] discard WEP mgt frame 'cuz "
+					"PRIVACY off\n",
+					ether_sprintf(wh->i_addr2)));
+				ic->ic_stats.is_rx_noprivacy++;
+				goto out;
+			}
+			key = ieee80211_crypto_decap(ic, ni, skb);
+			if (key == NULL) {
+				/* NB: stats+msgs handled in crypto_decap */
+				goto out;
+			}
+		}
 		(*ic->ic_recv_mgmt)(ic, skb, ni, subtype, rssi, rstamp);
 		dev_kfree_skb(skb);
 		return;
@@ -1356,6 +1388,7 @@ ieee80211_parse_rsn(struct ieee80211com *ic, u_int8_t *frm, struct ieee80211_rsn
 			("%s: length %u too short\n", __func__, len));
 		return IEEE80211_REASON_IE_INVALID;
 	}
+	frm += 2;		/* skip id+len */
 	w = LE_READ_2(frm);
 	if (w != RSN_VERSION) {
 		IEEE80211_DPRINTF(ic, IEEE80211_MSG_ELEMID | IEEE80211_MSG_WPA,
@@ -1727,7 +1760,7 @@ ieee80211_recv_mgmt(struct ieee80211com *ic, struct sk_buff *skb,
 		IEEE80211_ADDR_COPY(ni->ni_bssid, wh->i_addr3);
 		ni->ni_rssi = rssi;
 		ni->ni_rstamp = rstamp;
-		memcpy(ni->ni_tstamp, tstamp, sizeof(ni->ni_tstamp));
+		memcpy(ni->ni_tstamp.data, tstamp, sizeof(ni->ni_tstamp));
 		ni->ni_intval = bintval;
 		ni->ni_capinfo = capinfo;
 		/* XXX validate channel # */
