@@ -33,7 +33,7 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGES.
  *
- * $Id: ieee80211_wireless.c,v 1.17 2004/08/20 21:57:24 kcyu Exp $
+ * $Id: ieee80211_wireless.c,v 1.18 2004/09/15 23:11:26 samleffler Exp $
  */
 #ifndef EXPORT_SYMTAB
 #define	EXPORT_SYMTAB
@@ -334,8 +334,10 @@ ieee80211_ioctl_siwrate(struct ieee80211com *ic,
 	ifr.ifr_media = ic->ic_media.ifm_cur->ifm_media &~ IFM_TMASK;
 	if (rrq->fixed) {
 		/* XXX fudge checking rates */
-		rate = ieee80211_rate2media(ic, 2 * (rrq->value / 1000000),
+		rate = ieee80211_rate2media(ic, 2 * rrq->value / 1000000,
 				ic->ic_curmode);
+		if (rate == IFM_AUTO)		/* NB: unknown rate */
+			return -EINVAL;
 	} else
 		rate = IFM_AUTO;
 	ifr.ifr_media |= IFM_SUBTYPE(rate);
